/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.model;

import com.vividsolutions.jts.geom.*;
import com.vividsolutions.jts.geom.impl.CoordinateArraySequence;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import javax.swing.Action;
import nl.cloudfarming.client.fleet.model.gps.GPSAction;
import nl.cloudfarming.client.geoviewer.DynamicPoint;

/**
 * This class implements the geo functionality for machines. It provides methods
 * for changing the machine's location, handling property changes and all geo
 * related functionality.
 *
 * @author Wytse Visser
 */
public abstract class MachineGeometrical implements Serializable, DynamicPoint, PropertyChangeListener {

    private Machine machine;
    private PropertyChangeSupport propertyChangeSupport;
    private transient List<Action> mapActions = null;

    public MachineGeometrical(Machine machine) {
        this.machine = machine;
    }

    @Override
    public Point getCentroid() {
        Coordinate[] coordinates = new Coordinate[]{new Coordinate(machine.getLongitude(), machine.getLatitude())};
        CoordinateSequence coordSequence = new CoordinateArraySequence(coordinates);
        return new Point(coordSequence, new GeometryFactory());
    }

    @Override
    public String getTooltipText() {
        return machine.toString();
    }

    @Override
    public Point getRenderObject(Envelope envelope) {
        return getCentroid();
    }

    @Override
    public Geometry getBoundingBox() {
        return null;
    }

    @Override
    public String getIconLabel() {
        return machine.getName();
    }

    @Override
    public Point getGeometry() {
        return getCentroid();
    }

    @Override
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        machine.addPropertyChangeListener(this);
        getPropertyChangeSupport().addPropertyChangeListener(listener);
    }

    @Override
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        machine.removePropertyChangeListener(this);
        getPropertyChangeSupport().removePropertyChangeListener(listener);
    }

    private PropertyChangeSupport getPropertyChangeSupport() {
        if (propertyChangeSupport == null) {
            propertyChangeSupport = new PropertyChangeSupport(this);
        }
        return propertyChangeSupport;
    }

    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        switch (evt.getPropertyName()) {
            case Machine.PROP_LONGITUDE:
            case Machine.PROP_LATITUDE:
            case Machine.PROP_POSITION:
            case Machine.PROP_NAME:
                getPropertyChangeSupport().firePropertyChange(evt);
                break;
            default:
            // This PropertyChangeEvent has nothing to do with a geographical property, do nothing.
        }
    }

    @Override
    public void setPosition(double longitude, double latitude) {
        machine.setPosition(longitude, latitude);
    }

    @Override
    public void setGeometry(Point geometry) {
        setPosition(geometry.getCoordinate().x, geometry.getCoordinate().y);
    }

    @Override
    public Action[] getMapActions() {
        List<Action> actions = getActions();
        actions.add(GPSAction.get(machine));

        return getActions().toArray(new Action[0]);
    }

    private List<Action> getActions() {
        if (mapActions == null) {
            mapActions = new ArrayList<>();
        }
        return mapActions;
    }

    @Override
    public void addMapAction(Action action) {
        getActions().add(action);
    }
}
