/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.model.gps;

import com.vividsolutions.jts.geom.Point;
import nl.cloudfarming.client.fleet.model.Machine;
import org.openide.util.Exceptions;

/**
 * This class can be use superclass for implementations of GPS trackers. It
 * provides functions for enabling and disabling the tracker. Implementations
 * should override the getPosition() method.
 *
 * @author Wytse Visser
 */
public abstract class GPSTracker implements Runnable {

    private Thread thread = null;
    private Machine machine;
    private boolean enabled = false;
    protected static final int UPDATES_PER_SECOND = 1;
//    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.fleet.model.gps.GPSTracker");

    public void setMachine(Machine machine) {
        this.machine = machine;
    }

    /**
     * Toggles the gps tracker. If it's currently enabled, disable it and the
     * other way around.
     */
    public final void toggle() {
        if (isEnabled()) {
            disable();
        } else {
            enable();
        }
    }

    private void disable() {
        setEnabled(false);
    }

    private void enable() {
        if (thread == null) {
            thread = new Thread(this);
            thread.start();
        }
        setEnabled(true);
    }

    /**
     *
     * @return Whether the gps tracker is enabled.
     */
    protected boolean isEnabled() {
        return enabled;
    }

    private void setEnabled(boolean enabled) {
        this.enabled = enabled;
    }

    /**
     *
     * @return The machine this gps tracker is tracking for.
     */
    protected Machine getMachine() {
        return machine;
    }

    /**
     * This method runs the gps tracker. It starts a loop in which it sleeps for
     * a while (based on UPDATES_PER_SECOND) and then updates the position of
     * the machine.
     */
    @Override
    public void run() {
        while (true) {
            try {
                Thread.sleep(1000 / UPDATES_PER_SECOND);
            } catch (InterruptedException ex) {
                Exceptions.printStackTrace(ex);
            }
            if (isEnabled()) {
                Point position = getPosition();
                getMachine().setLongitude(position.getX());
                getMachine().setLatitude(position.getY());
            }
        }
    }

    /**
     * Get the current position of the machine.
     *
     * @return The current position of the machine.
     */
    protected abstract Point getPosition();
}
